import { loadCss, loadScript } from '@cnblogs/load-script-css';
import { defaultConfig, MathEngine, } from './model';
export class Mathjax2Renderer {
    _defaultConfig = defaultConfig[MathEngine.MathJax2];
    _defaultMathJaxConfig = {
        tex2jax: {
            inlineMath: [
                ['$', '$'],
                ['\\(', '\\)'],
            ],
            processClass: 'math',
            processEscapes: true,
        },
        TeX: {
            equationNumbers: {
                autoNumber: 'AMS',
                useLabelIds: true,
            },
            extensions: ['extpfeil.js', 'mediawiki-texvc.js'],
            Macros: { bm: '\\boldsymbol' },
        },
        CommonHTML: { linebreaks: { automatic: true } },
        'HTML-CSS': { linebreaks: { automatic: true } },
        SVG: { linebreaks: { automatic: true } },
        skipStartupTypeset: true,
    };
    _config = this._defaultConfig;
    _initTask;
    get _global() {
        return (this._config.document.defaultView ?? window);
    }
    render(container, callback) {
        if (!this._config.isEnabled)
            return Promise.resolve();
        return this.init()
            .then(mathjax => {
            if (!mathjax)
                return;
            const { onRendered } = this._config;
            mathjax.Hub.Queue(['Typeset', mathjax.Hub, container]);
            if (onRendered != null) {
                mathjax.Hub.Queue(() => onRendered(mathjax));
            }
            if (callback) {
                mathjax.Hub.Queue(callback);
            }
        })
            .catch(console.warn);
    }
    config(value) {
        this._config = Object.assign({}, this._defaultConfig, value);
        return this;
    }
    init() {
        if (this._initTask)
            return this._initTask;
        const { entryScript, tapMathJaxConfig, document } = this._config;
        return (this._initTask = loadScript({
            src: entryScript,
            id: 'mathjax',
            timeoutSeconds: 10,
            appendTo: document.head,
        })
            .then(() => {
            const { MathJax } = this._global;
            if (MathJax) {
                MathJax.Hub.Config(tapMathJaxConfig?.call(undefined, Object.assign({}, this._defaultMathJaxConfig)) ?? {
                    ...this._defaultMathJaxConfig,
                });
                MathJax.Hub.Configured();
                return MathJax;
            }
        })
            .catch(() => (this._initTask = undefined)));
    }
}
export class MathJax3Renderer {
    _defaultConfig = defaultConfig[MathEngine.MathJax3];
    _defaultMathJaxConfig = {
        tex: {
            inlineMath: [
                ['$', '$'],
                ['\\(', '\\)'],
            ],
            processEscapes: true,
            tags: 'ams',
            useLabelIds: true,
            macros: {
                bm: '\\boldsymbol',
            },
            packages: { '[+]': ['noerrors', 'extpfeil'] },
        },
        options: {
            processHtmlClass: 'math',
            ignoreHtmlClass: 'tex2jax_ignore',
        },
        loader: {
            load: ['[tex]/noerrors', '[tex]/extpfeil'],
        },
        startup: {
            typeset: false,
        },
    };
    _config = this._defaultConfig;
    _initTask;
    config(value) {
        this._config = Object.assign({}, this._defaultConfig, value);
        return this;
    }
    render(container, callback) {
        if (!this._config.isEnabled)
            return Promise.resolve();
        return this.init()
            .then(mathjax => {
            mathjax
                ? (mathjax.startup.promise = mathjax.startup.promise.then(() => mathjax.typesetPromise([container]).then(() => {
                    try {
                        this._config.onRendered?.call(undefined, mathjax);
                    }
                    catch (e) {
                        console.error(e);
                    }
                    try {
                        callback?.call(undefined);
                    }
                    catch (e) {
                        console.error(e);
                    }
                })))
                : undefined;
        })
            .catch(console.warn);
    }
    init() {
        return (this._initTask ??= new Promise((resolve, reject) => {
            const { entryScript, tapMathJaxConfig, document: _document } = this._config;
            const _window = (_document.defaultView ?? window);
            _window.MathJax = tapMathJaxConfig?.call(undefined, {
                ...this._defaultMathJaxConfig,
            }) ?? { ...this._defaultMathJaxConfig };
            if (!_window.MathJax.startup) {
                _window.MathJax.startup = {};
            }
            _window.MathJax.startup.ready = () => {
                _window.MathJax?.startup?.defaultReady();
                resolve(_window.MathJax);
            };
            loadScript({
                src: entryScript,
                id: 'mathjax',
                executionStrategy: 'defer',
                appendTo: _window.document.head,
            }).catch(reject);
        })).catch(() => (this._initTask = undefined));
    }
}
export class KatexRenderer {
    _defaultConfig = defaultConfig[MathEngine.Katex];
    _defaultKatexOptions = {
        delimiters: [
            { left: "$$", right: "$$", display: true },
            { left: "$", right: "$", display: false },
            { left: "\\(", right: "\\)", display: false },
            { left: "\\[", right: "\\]", display: true }
        ],
    };
    _config = this._defaultConfig;
    _katexOption = Object.assign({}, this._defaultKatexOptions);
    _initTask;
    get _global() {
        return (this._config.document.defaultView ?? window);
    }
    config(value) {
        this._config = Object.assign({}, this._defaultConfig, value);
        return this;
    }
    async render(container, callback) {
        if (!this._config.isEnabled) {
            return Promise.resolve();
        }
        await this.init();
        const global = this._global;
        global.renderMathInElement(container, this._katexOption);
        if (callback) {
            callback();
        }
    }
    init() {
        return (this._initTask ??= new Promise((resolve, reject) => {
            const { entryScript, document: _document } = this._config;
            Promise.all([
                loadCss({
                    href: 'https://common.cnblogs.com/katex/0.16.8/katex.min.css',
                    id: 'katex-style',
                    appendTo: _document.head
                }),
                loadScript({
                    src: entryScript,
                    id: 'katex',
                    appendTo: _document.head,
                })
            ]).then(() => loadScript({
                src: 'https://common.cnblogs.com/katex/0.16.8/contrib/auto-render.min.js',
                id: 'katex-auto-render',
                appendTo: _document.head,
            }).then(() => {
                this._katexOption = this._config.tapMathJaxConfig?.call(undefined, this._katexOption) ?? {
                    ...this._defaultKatexOptions,
                };
                resolve();
            })).catch(reject);
        })).catch(() => (this._initTask = undefined));
    }
}
//# sourceMappingURL=impl.js.map